#
#  activity_spots.py
#  
#
#  Created by Dumusque Xavier on 2/17/09.
#  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
#

# Generating spot of maximal size according to the observed distribution on the Sun. The lifetime of spot are then different. The apparition of spots is ruled by a Poisson law

import matplotlib
import matplotlib.animation as manimation
from pylab import *
import os,sys,string
import glob
import random
import shutil
import pyfits
import math

sys.path.append('functions/')

import pyrdb
import periodogram

#######################################################################################
###                                Parameters                                       ###
#######################################################################################

K_planet = 3. # in m/s
P_planet = 30. # in days

#######################################################################################
###                                Periodogram                                      ###
#######################################################################################

## GLS, Zechmeister 2009
def compute_periodogram(x,y,sig_y=0,number_of_night=1,ofac=1.):
    
    ## x and y must be float vectors
    x = array(x)*1.
    y = array(y)*1.
    
    if type(sig_y) == int:
        weight=ones(len(x))/len(x)
        W = sum(weight)
    else:
        W = sum(1./array(sig_y)**2)
        weight=1./W * 1./array(sig_y)**2

    median_step=median(x[1:]-x[:-1])
    hifac = median_step

    freq,power,phi,a_cos,b_sin,c_cte,fmax,fap,z0,z01,variance = periodogram.periodogram(x,y,weight,ofac,hifac,number_of_night)
    
    return freq,power,phi

#######################################################################################
###                                Read data                                        ###
#######################################################################################

file = 'results/25_05_incli_90/log_Rhk_48/spot_and_plage_vrad_effect_log_Rhk_48.rdb'
data = pyrdb.read_rdb(file)
for i in arange(len(data.keys())):
    exec data.keys()[i]+" = array(data[data.keys()[i]])"

RV_planet = K_planet*sin(2*pi/P_planet*time)

RV = vrad_spot + vrad_plage + RV_planet
FWHM = fwhm_spot + fwhm_plage

#RV FWHM correlation
fit_RV_FWHM = polyfit(FWHM,RV,1)

figure(0)
title('RV FWHM correlation')
plot(FWHM,RV,'o')
plot(FWHM,poly1d(fit_RV_FWHM)(FWHM),lw=3,color='r')
xlabel('FWHM [m/s]')
ylabel('RV [m/s]')

figure(1)
RV_corrected_with_fwhm = RV - poly1d(fit_RV_FWHM)(FWHM)
subplot(211)
title('raw RVs')
plot(time,RV,label='RMS = %.2f'%std(RV))
ylabel('raw RV\n[m/s]')
legend()
subplot(212)
title('RV corrected with FWHM')
plot(time,RV_corrected_with_fwhm,label='RMS = %.2f'%std(RV_corrected_with_fwhm))
ylabel("RV corrected\nusing FWHM [m/s]")
legend()
xlabel('time [d]')
subplots_adjust(top=0.95,left=0.1,bottom=0.08,right=0.98,wspace=0.32,hspace=0.32)
savefig('figures/RV_corrected_using_FWHM.pdf')

perio = compute_periodogram(time,RV,ones(len(RV)),ofac=4.)
perio_corrected = compute_periodogram(time,RV_corrected_with_fwhm,ones(len(RV_corrected_with_fwhm)),ofac=4)

figure(2)
subplot(211)
title('Periodogram RV not corrected')
axvline(P_planet,color='r',ls='--',lw=3,zorder=0)
semilogx(1./perio[0],perio[1],zorder=2)
ylabel('Normalized Power')
ax=gca()
subplot(212,sharex=ax)
title('Periodogram RV corrected with FWHM')
axvline(P_planet,color='r',ls='--',lw=3,zorder=0)
semilogx(1./perio_corrected[0],perio_corrected[1],zorder=2)
ylabel('Normalized Power')
xlabel('Period [d]')
xlim(5,500)

savefig('figures/Periodogram_RV_corrected_using_FWHM.pdf')


